#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    plibraryloader( new Libraryloader)
{
    ui->setupUi(this);

    setFixedSize(width(),height());
#ifdef __linux__
    //QString name="/home/jonti/workspace/JMPXEncoder/Release/libjmpx.so.1";
    QString name="libjmpx.so.1";
    pMPX=plibraryloader->LoadLibrary(name);
    if(!pMPX)
    {
        pMPX=plibraryloader->LoadLibrary(QCoreApplication::applicationDirPath()+"/libjmpx.so.1");
    }
#else
    pMPX=plibraryloader->LoadLibrary(QCoreApplication::applicationDirPath()+"/libjmpx1.dll");
#endif
    if(pMPX)
    {
        pMPX->SetPreEmphasis(WORLD);
        SDevices* pdev=pMPX->GetDevices();
        ui->comboBox->clear();
        for(int i=0;i<pdev->NumberOfDevices;i++)
        {
            ui->comboBox->addItem(pdev->Device[i].name);
        }
        if(!pdev->NumberOfDevices)ui->comboBox->addItem("None");
    }
     else QMessageBox::critical(this,"Error","<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Unable to load MPX library.</span></p><p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">library file MPX needs to be placed in the gui's program directory.</p>");
    ptimer= new QTimer(this);
    ptimer->setInterval(20);
    connect(ptimer,SIGNAL(timeout()),this,SLOT(updatedisplay()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_checkBox_stateChanged(int state)
{
    if(!pMPX)return;
    if((!pMPX->IsActive())&&(state))
    {
        pMPX->SetSampleRate(192000);
        pMPX->SetSoundCard(ui->comboBox->currentIndex());
        pMPX->SetBufferFrames(8096);
    }

    pMPX->Active(state);
    if((state)&&(pMPX->GotError()))
    {
        QMessageBox msgBox;
        msgBox.setText(pMPX->GetLastRTAudioError());
        msgBox.exec();
        state=false;
    }
    pMPX->EnableStereo(ui->checkBox_2->isChecked());

    if(ui->radionone->isChecked())pMPX->SetPreEmphasis(NONE);
     else if(ui->radio50->isChecked())pMPX->SetPreEmphasis(WORLD);
      else if(ui->radio75->isChecked())pMPX->SetPreEmphasis(USA);

    if(state)ptimer->start();
     else ptimer->stop();
     ui->lvolumemeter->setVolume(0);
     ui->rvolumemeter->setVolume(0);
     ui->outvolumemeter->setVolume(0);
}

void MainWindow::on_checkBox_2_stateChanged(int state)
{
    if(!pMPX)return;
    pMPX->EnableStereo(state);
}

void MainWindow::updatedisplay()
{
    if(!pMPX)return;
    TSigStats *psigstats=pMPX->GetSignalStats();
    ui->lvolumemeter->setVolume(psigstats->lvol);
    ui->rvolumemeter->setVolume(psigstats->rvol);
    ui->outvolumemeter->setVolume(psigstats->outvol);
}

void MainWindow::on_radio50_clicked()
{
    if(!pMPX)return;
    pMPX->SetPreEmphasis(WORLD);
}

void MainWindow::on_radio75_clicked()
{
    if(!pMPX)return;
    pMPX->SetPreEmphasis(USA);
}

void MainWindow::on_radionone_clicked()
{
    if(!pMPX)return;
    pMPX->SetPreEmphasis(NONE);
}

void MainWindow::on_comboBox_currentIndexChanged(int index)
{
    if(!pMPX)return;
    pMPX->SetSoundCard(index);
}
