//---------------------------------------------------------------------------


#include "JDSP.hpp"

//---------------------------------------------------------------------------



 WaveTable::WaveTable(TDspGen *_pDspGen) :
        pDspGen(_pDspGen)
{
        RefreshSettings();
}



WaveTable::~WaveTable()
{
 //
}

void  WaveTable::RefreshSettings()
{
        WTstep=(pDspGen->Freq)*(WTSIZE)/((double)(pDspGen->SampleRate));
        WTptr=0;
        intWTptr=0;
}

void WaveTable::WTnextFrame()
{
	WTptr+=WTstep;
	while(!signbit(WTptr-(double)WTSIZE))WTptr-=(double)(WTSIZE);
	if(signbit(WTptr))WTptr=0;
	intWTptr=(int)WTptr;intWTptr%=WTSIZE;
}

double   WaveTable::WTSinValue()
{
	return pDspGen->SinWT[intWTptr];
}

double   WaveTable::WTSin2Value()
{
        double tmpwtptr=WTptr*2.0;
    	while(!signbit(tmpwtptr-(double)WTSIZE))tmpwtptr-=(double)(WTSIZE);
    	if(signbit(tmpwtptr))tmpwtptr=0;
		return pDspGen->SinWT[(int)tmpwtptr];
}
//----------

void PreEmphasis::SetTc(TimeConstant timeconst)
{
	switch(timeconst)
	{
	case WORLD:
	    a[0] = 5.309858008l;
	    a[1] = -4.794606188l;
	    b[1] = 0.4847481783l;
		break;
	case USA:
        a[0] = 7.681633687l;
        a[1] = -7.170926091l;
        b[1] = 0.4892924010l;
		break;
	default:
        a[0] = 1.0l;
        a[1] = 0.0l;
        b[1] = 0.0l;
		break;
	}


	y[0]=0;
	y[1]=0;
	x[0]=0;
	x[1]=0;
}

PreEmphasis::PreEmphasis()
{
	SetTc(WORLD);
}

double PreEmphasis::Update(double val)
{
	x[1]=val;
	y[1]=a[0]*x[1]+a[1]*x[0]+b[1]*y[0];
	double retval=y[1];
	y[0]=y[1];
	x[0]=x[1];
	return retval;
}

Clipper::Clipper()
{
	SetCompressionPoint(0.85l);
}

void Clipper::SetCompressionPoint(double point)
{
	if(point>0.99l)point=0.99l;
	 else if(point<0.01l)point=0.01l;
	compressionpoint=point;
	double a=M_PI/(2.0l*(1.0l-compressionpoint));
	LookupTable.resize(6000);
	for(int i=0;i<(int)LookupTable.size();i++)
	{
		LookupTable[i]=compressionpoint+atan(a*(i*0.001l))/a;
	}
}

double Clipper::Update(double val)
{
	if(fabs(val)<compressionpoint)return val;
	double inv=1.0l;
	if(val<0.0l)
	{
		inv=-1.0l;
		val=-val;
	}

	double shiftmultval=(val-compressionpoint)*1000.0l;
	int n=(int)(shiftmultval);
	int q=n+1;
	if(q>=(int)LookupTable.size())return inv;
	if(n<0)return compressionpoint*inv;
	val=LookupTable[n]+(shiftmultval-((double)n))*(LookupTable[q]-LookupTable[n]);
	return inv*val;
}

FIRFilter::FIRFilter()
{
	BufPtr=0;
}

double FIRFilter::Update(double val)
{
	if(Buf.size()!=Cof.size())
	{
		Buf.resize(Cof.size());
		BufPtr=0;
	}
	BufPtr%=Buf.size();
	Buf[BufPtr]=val;
	BufPtr++;BufPtr%=Buf.size();
	double retval=0.0l;
	for(unsigned int i=0;i<Cof.size();i++)
	{
		retval+=Buf[BufPtr]*Cof[i];
		BufPtr++;BufPtr%=Buf.size();
	}
	return retval;
}


//-----


 TDspGen::TDspGen(TSetGen *_pSetGen) :
        pSetGen(_pSetGen)
{
        SinWT.resize(WTSIZE);
        int i;
        for(i=0;i<WTSIZE;i++){SinWT[i]=(sin(2.0*M_PI*((double)i)/((double)WTSIZE)));}
        ResetSettings();
}

 TDspGen::~TDspGen()
{
//
}

void  TDspGen::ResetSettings()
{
        //load values
        SampleRate=pSetGen->SampleRate;
        Freq=pSetGen->Freq;
}

