
#include "JMPXEncoder.hpp"



JMPXEncoder::JMPXEncoder()	:
	pTDspGen( new TDspGen(&ASetGen)),
	pWaveTable( new WaveTable(pTDspGen.get()))
{
	stereo=true;
	decl=0;
	lbigval=0.0l;
	rbigval=0.0l;
	outbigval=0.0l;
	sigstats.lvol=0.0l;
	sigstats.rvol=0.0l;
	sigstats.outvol=0.0l;

	AJCSound.sampleRate=192000;
	AJCSound.bufferFrames=8096;
}

JMPXEncoder::~JMPXEncoder()
{
        AJCSound.Active(false);
        AJCSound.FOnSoundEvent=NULL;
}



void JMPXEncoder::Active(bool Enabled)
{
        if(Enabled==AJCSound.IsActive())return;
        if(Enabled)
        {

            AJCSound.FOnSoundEvent=NULL;

            ASetGen.SampleRate=AJCSound.sampleRate;
            ASetGen.Freq=19000.0;

            pTDspGen->ResetSettings();
            pWaveTable->RefreshSettings();

            lpreemp.SetTc(PreEmphasis::WORLD);
            rpreemp.SetTc(PreEmphasis::WORLD);

            AJCSound.FOnSoundEvent=boost::bind( boost::mem_fn( &JMPXEncoder::Update ) ,this,_1,_2,_3);

        }
        AJCSound.Active(Enabled);
}

void JMPXEncoder::EnableStereo(bool enable)
{
	stereo=enable;
}

TSigStats* JMPXEncoder::GetSignalStats()
{
	return &sigstats;
}

void JMPXEncoder::SetPreEmphasis(TimeConstant timeconst)
{
	lpreemp.SetTc((PreEmphasis::TimeConstant)timeconst);
	rpreemp.SetTc((PreEmphasis::TimeConstant)timeconst);
}

void JMPXEncoder::Update(double *DataIn,double *DataOut, int Size)
{
	double rval;
	double lval;

	static Signal<double> laudio_signal(10240);
	static basic_FIR<double> lfir (Low_pass (16500, ASetGen.SampleRate, 60));
	static Signal<double> raudio_signal(10240);
	static basic_FIR<double> rfir (Low_pass (16500, ASetGen.SampleRate, 60));

	if(!stereo)
	{
		for(int i=0;i<(Size-1);i+=2)
		{
			laudio_signal << DataIn[i];
			raudio_signal << DataIn[i+1];
			lval=laudio_signal.filtered_output(lfir);
			rval=raudio_signal.filtered_output(rfir);

			lval=lpreemp.Update(lval*0.5l);
			rval=rpreemp.Update(rval*0.5l);

			lval=clipper.Update(lval);
			rval=clipper.Update(rval);

			DataOut[i]=0.5*(lval+rval);
			DataOut[i+1]=DataOut[i];

			if(fabs(DataIn[i])>lbigval)lbigval=fabs(DataIn[i]);
			if(fabs(DataIn[i+1])>rbigval)rbigval=fabs(DataIn[i+1]);
			if(fabs(DataOut[i])>outbigval)outbigval=fabs(DataOut[i]);
			decl++;
			if(!(decl%=5000))
			{
				sigstats.lvol=(1-0.2l)*sigstats.lvol+(0.2l)*lbigval;
				sigstats.rvol=(1-0.2l)*sigstats.rvol+(0.2l)*rbigval;
				sigstats.outvol=(1-0.2l)*sigstats.outvol+(0.2l)*outbigval;
				decl=0;
				lbigval=0.0l;
				rbigval=0.0l;
				outbigval=0.0l;
			}

		}
		return;
	}


	for(int i=0;i<(Size-1);i+=2)
	{
		pWaveTable->WTnextFrame();

		laudio_signal << DataIn[i];
		raudio_signal << DataIn[i+1];
		lval=laudio_signal.filtered_output(lfir);
		rval=raudio_signal.filtered_output(rfir);

		lval=lpreemp.Update(lval*0.5l);
		rval=rpreemp.Update(rval*0.5l);

		lval=clipper.Update(lval);
		rval=clipper.Update(rval);

		DataOut[i]=0.45*(lval+rval);
		DataOut[i]+=0.09l*pWaveTable->WTSinValue();
		DataOut[i]+=0.45l*(pWaveTable->WTSin2Value())*(lval-rval);

		if(fabs(DataIn[i])>lbigval)lbigval=fabs(DataIn[i]);
		if(fabs(DataIn[i+1])>rbigval)rbigval=fabs(DataIn[i+1]);
		if(fabs(DataOut[i])>outbigval)outbigval=fabs(DataOut[i]);
		decl++;
		if(!(decl%=5000))
		{
			sigstats.lvol=(1-0.2l)*sigstats.lvol+(0.2l)*lbigval;
			sigstats.rvol=(1-0.2l)*sigstats.rvol+(0.2l)*rbigval;
			sigstats.outvol=(1-0.2l)*sigstats.outvol+(0.2l)*outbigval;
			decl=0;
			lbigval=0.0l;
			rbigval=0.0l;
			outbigval=0.0l;
		}

		DataOut[i+1]=DataOut[i];
	}
}



// the class factories

extern "C"
{
	JMPXInterface* create()
	{
		return new JMPXEncoder;
	}

	void destroy(JMPXInterface* p)
	{
		delete p;
	}
}

